import React, { useEffect, useState } from 'react'
import { publicFetch } from '../../util/fetcher'
import AutomatedAnswer from '../automated-answer'
import NoAutomatedAnswer from '../automated-answer/no-automated-answer'
import { Spinner } from '../icons'
import { useInterval } from '../../hooks/useInterval'

import styles from './automated-answer-container.module.css'

const AutomatedAnswerContainer = ({ question_id }) => {
  const [loading, setLoading] = useState(true)
  const [polled, setPolled] = useState(500)
  const [tries, setTries] = useState(5)
  const [answer, setAnswer] = useState()
  const [isAnswerGenerating, setIsAnswertGenerating] = useState(false)
  const [error, setError] = useState({
    status: '',
    action: '',
    button: '',
    message: ''
  })

  const fetchAutomatedAnswer = async () => {
    if (tries > 0) {
      setTries(tries - 1)
      setPolled(null)
    }
    try {
      const response = await publicFetch.get(`automatedanswer/${question_id}`)
      if (response.status == 200) {
        console.log('RESPONSE IS HERE AND IT IS')
        const { isLoading, isComplete } = response.data.automatedanswer
        if (isLoading && !isComplete) {
          setIsAnswertGenerating(true)
          setServerError(102)
          setPolled(1000)
        } else {
          setIsAnswertGenerating(false)
          setAnswer(response.data)
          setPolled(null)
        }
      } else setServerError({ response })
    } catch (error) {
      setServerError(error)
    }
    setLoading(false)
  }

  useInterval(async () => {
    console.log('Checking')

    await fetchAutomatedAnswer()
  }, polled)

  const setServerError = (error) => {
    if (error == 102) {
      setError({
        status: '102',
        action: 'loading',
        button: 'Answer is generatingg',
        message: 'Automated answer 🤖 is generating in the backend '
      })
    } else if (error.response.status == 404) {
      setError({
        status: '404',
        action: 'create',
        button: 'Generate One',
        message: 'No automated answer found for this question'
      })
    } else {
      setError({
        status: '500',
        action: 'tryagain',
        button: 'Try again',
        message: 'Server error, Please try again'
      })
    }
  }

  const getBody = () => {
    if (loading) {
      return <Spinner className={styles.spinner} />
    } else if (isAnswerGenerating) {
      return (
        <NoAutomatedAnswer
          question_id={question_id}
          error={error}
          isGenerating={true}
          bindIsGenerating={setIsAnswertGenerating}
        />
      )
    } else if (answer != null) {
      return <AutomatedAnswer automatedAnswer={answer.automatedanswer} />
    } else {
      return (
        <NoAutomatedAnswer
          question_id={question_id}
          error={error}
          isGenerating={false}
          bindIsGenerating={setIsAnswertGenerating}
        />
      )
    }
  }

  return (
    <div className={styles.container}>
      <div className={styles.header}>
        <div className={styles.fill}>
          <h2>Automated Answer</h2>
        </div>
      </div>
      <div className={styles.wrapper}>{getBody(answer)}</div>
    </div>
  )
}

export default AutomatedAnswerContainer
